/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use,   *
 * modify, and distribute the following source code (the "Software")     *
 * on any number of computers. You may use the Software as part of       *
 * creating a software program or product intended for commercial or     *
 * non-commercial distribution in machine-readable source code, binary,  *
 * or executable formats. You may distribute the Software as             *
 * machine-readable source code provided this license is not removed     *
 * from the Software and any modifications are conspicuously indicated.  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not remove any copyright, trademark, or other proprietary     *
 * notices from the Software or the associated  documentation.           *
 * You are responsible for compliance with all applicable export or      *
 * re-export control laws and regulations if you export the Software.    *
 * This license is governed by and is to be construed under the laws     *
 * of the State of Texas.                                                *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */

/*
** Verify that the passed file is an image by opening it and reading in 
** an "image header" - if it matches what we expect, return 1, otherwise 
** return 0.
*/

#include "vms_jackets.h"
#include <stdio.h>

/*
** We want to use the special jacketted fopen.
*/
#define fopen GENERIC_FOPEN_JACKET
FILE *fopen(const char *__filename, const char *__mode, ...);

#if defined __ALPHA

#define EIHD$K_MAJORID 3
#define EIHD$K_MINORID 0
#define EIHD$K_EXE 1

typedef struct _eihd {
    struct  {
        unsigned int eihd$l_majorid;
        unsigned int eihd$l_minorid;
        } eihd$r_version;
    unsigned int eihd$l_size;
    unsigned int eihd$l_isdoff;
    unsigned int eihd$l_activoff;
    unsigned int eihd$l_symdbgoff;
    unsigned int eihd$l_imgidoff;
    unsigned int eihd$l_patchoff;
    union  {
        void *eihd$l_iafva;
        __int64 eihd$q_iafva;
        } eihd$r_iafva;
    union  {
        void *eihd$l_symvva;
        __int64 eihd$q_symvva;
        } eihd$r_symvva;
    unsigned int eihd$l_version_array_off;
    unsigned int eihd$l_imgtype;
    unsigned int eihd$l_subtype;
    unsigned int eihd$l_imgiocnt;
    unsigned int eihd$l_iochancnt;
    unsigned __int64 eihd$q_privreqs;
    unsigned int eihd$l_hdrblkcnt;
    unsigned int eihd$l_lnkflags;
    unsigned int eihd$l_ident;
    unsigned int eihd$l_sysver;
    struct  {
        unsigned char eihd$b_matchctl;
        unsigned char eihd$b_fill_1;
        unsigned char eihd$b_fill_2;
        unsigned char eihd$b_fill_3;
        unsigned int eihd$l_symvect_size;
        unsigned int eihd$l_virt_mem_block_size;
        } eihd$r_control;
    char eihd$t_skip [406];
    unsigned short int eihd$w_alias;
    } EIHD;
#else
			/* VAX */
#define IHD$K_MAJORID 12848
#define IHD$K_MINORID 13616
#define IHD$K_EXE 1
typedef struct _ihd {
    unsigned short int ihd$w_size;
    unsigned short int ihd$w_activoff;
    unsigned short int ihd$w_symdbgoff;
    unsigned short int ihd$w_imgidoff;
    unsigned short int ihd$w_patchoff;
    unsigned short int ihd$w_version_array_off;
    unsigned short int ihd$w_majorid;
    unsigned short int ihd$w_minorid;
    unsigned char ihd$b_hdrblkcnt;
    unsigned char ihd$b_imgtype;
    short int ihddef$$_fill_2;
    unsigned int ihd$q_privreqs [2];
    unsigned short int ihd$w_iochancnt;
    unsigned short int ihd$w_imgiocnt;
    unsigned int ihd$l_lnkflags;
    unsigned int ihd$l_ident;
    unsigned int ihd$l_sysver;
    void *ihd$l_iafva;
    char ihd$t_skip [462];
    unsigned short int ihd$w_alias;
} IHD;
#endif


int GENERIC_IS_IMAGE (char *file_name)
/*
** Open "file_name", and check to see if its an image.
** Return 1 if it is, all other cases (such as the file does not exist or
** failed to read it) return 0
*/
{
FILE 
   *a_file;

#if defined __ALPHA
EIHD
   image_header;
#else
IHD
   image_header;
#endif

    if ( (a_file = fopen(file_name, "r")) == NULL) {
	return 0;
    }

    if (fread (&image_header, sizeof (image_header), 1, a_file) != 1 ) {
	fclose (a_file);
	return 0;
    }
    /* done with it */
    fclose (a_file);

#if defined __ALPHA

    if ( (image_header.eihd$r_version.eihd$l_majorid <= EIHD$K_MAJORID) && 
#if EIHD$K_MINORID == 0
	  (image_header.eihd$r_version.eihd$l_minorid == EIHD$K_MINORID) &&
#else
	  (image_header.eihd$r_version.eihd$l_minorid <= EIHD$K_MINORID) &&
#endif
	  (image_header.eihd$l_imgtype == EIHD$K_EXE) ) {
	return 1;
    }
#else
    if ( (image_header.ihd$w_majorid <= IHD$K_MAJORID) &&
	 (image_header.ihd$w_minorid <= IHD$K_MINORID) &&
	 (image_header.ihd$b_imgtype == IHD$K_EXE) &&
	 ( (image_header.ihd$w_alias == 0xffff) ||
	   (image_header.ihd$w_alias == 0x0002) ||
	   (image_header.ihd$w_alias == 0x0003) ) ) {
	return 1;
    }
#endif
    return 0;
}		/* End of rtn */
